"""Error handling."""

from contextlib import contextmanager
from typing import Optional, Iterator

from yaml import Mark, Node


class YamlenError(RuntimeError):
    def __init__(self, cause: Exception, mark: Optional[Mark] = None):
        self._cause = cause
        self._mark = mark

    def __str__(self) -> str:
        lines = []
        if self._cause:
            lines.append(str(self._cause))
        if self._mark:
            lines.append(str(self._mark))
        return "\n".join(lines)


@contextmanager
def on_node(node: Node) -> Iterator:
    try:
        yield
    except Exception as error:
        raise YamlenError(mark=node.start_mark, cause=error)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['yamlen', 'yamlen.tag', 'yamlen.tag.impl']

package_data = \
{'': ['*']}

install_requires = \
['pyyaml>=5.4.1,<6.0.0']

setup_kwargs = {
    'name': 'yamlen',
    'version': '0.1.1',
    'description': 'A PyYAML thin wrapper.',
    'long_description': '# Yamlen - a Thin PyYAML Wrapper\n\n[![CircleCI](https://circleci.com/gh/ymoch/yamlen.svg?style=svg)][Circle CI]\n[![Codecov](https://codecov.io/gh/ymoch/yamlen/branch/main/graph/badge.svg)][Codecov]\n\n## Features\n- Contextual tag construction.\n\n## Examples\n\n### Create a Loader\n```\n>>> from yamlen import Loader\n>>> loader = Loader()\n\n```\n\n\n### Load YAML documents in Streams\n\n```\n>>> from io import StringIO\n\n>>> stream = StringIO("foo")\n>>> loader.load(stream)\n\'foo\'\n\n>>> stream = StringIO("foo\\n---\\nbar")\n>>> list(loader.load_all(stream))\n[\'foo\', \'bar\']\n\n```\n\n### Load YAML Documents in Files.\n\n```\n>>> import os\n>>> from tempfile import TemporaryDirectory\n\n>>> with TemporaryDirectory() as dir_path:\n...     path = os.path.join(dir_path, "example.yml")\n...     with open(path, "w") as f:\n...         _ = f.write("foo")\n...     loader.load_from_path(path)\n\'foo\'\n\n>>> with TemporaryDirectory() as dir_path:\n...     path = os.path.join(dir_path, "example.yml")\n...     with open(path, "w") as f:\n...         _ = f.write("foo\\n---\\nbar")\n...     list(loader.load_all_from_path(path))\n[\'foo\', \'bar\']\n\n```\n\n### Contextual tag construction: include another YAML file.\n\n```\n>>> from yamlen.tag.impl.inclusion import InclusionTag\n>>> loader.add_tag("!include", InclusionTag())\n\n```\n\n```\n>>> with TemporaryDirectory() as dir_path:\n...     foo_path = os.path.join(dir_path, "foo.yml")\n...     bar_path = os.path.join(dir_path, "bar.yml")\n...     with open(foo_path, "w") as f:\n...         _ = f.write(f"!include ./bar.yml")\n...     with open(bar_path, "w") as f:\n...         _ = f.write("bar")\n...     loader.load_from_path(foo_path)\n\'bar\'\n\n```\n\n## License\n\n[![MIT License](https://img.shields.io/badge/License-MIT-brightgreen.svg)][MIT License]\n\nCopyright (c) 2021 Yu Mochizuki\n\n[Circle CI]: https://circleci.com/gh/ymoch/yamlen\n[Codecov]: https://codecov.io/gh/ymoch/yamlen\n[MIT License]: https://opensource.org/licenses/MIT\n',
    'author': 'Yu Mochizuki',
    'author_email': 'ymoch.dev@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/ymoch/yamlen',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



import glob
import os
import re

from yaml import Node, ScalarNode

from yamlen.loader import TagContext, Tag

__all__ = ("InclusionTag",)

_WILDCARDS_REGEX = re.compile(r"^.*(\*|\?|\[!?.+]).*$")


class InclusionTag(Tag):
    def construct(self, node: Node, context: TagContext) -> object:
        origin = context.origin
        if origin is None:
            raise ValueError(
                "cannot decide the target directory because no origin path is given"
            )

        if not isinstance(node, ScalarNode):
            raise ValueError(f"expected a scalar node, but found {node.tag}")

        base = context.constructor.construct_scalar(node)
        if not base:
            raise ValueError("given no path")

        path = os.path.join(origin, str(base))
        if _WILDCARDS_REGEX.match(path):
            paths = glob.iglob(path, recursive=True)
            return [context.loader.load_from_path(p) for p in paths]
        return context.loader.load_from_path(path)



"""Loaders."""

from __future__ import annotations

import os
from abc import ABC, abstractmethod
from contextlib import contextmanager
from functools import partial
from typing import Any, Iterator, Optional, TextIO

from yaml import Node, MarkedYAMLError
from yaml import load as _load, load_all as _load_all
from yaml.composer import Composer
from yaml.constructor import BaseConstructor, SafeConstructor
from yaml.parser import Parser
from yaml.reader import Reader
from yaml.resolver import Resolver
from yaml.scanner import Scanner

from .error import YamlenError, on_node

__all__ = ("Loader", "Tag", "TagContext")


class TagContext:
    """
    A tag construction context.
    """

    def __init__(
        self,
        loader: Loader,
        constructor: BaseConstructor,
        origin: Optional[str] = None,
    ):
        self._loader = loader
        self._constructor = constructor
        self._origin = origin

    @property
    def loader(self) -> Loader:
        """
        Returns a current YAML loader.
        """
        return self._loader

    @property
    def constructor(self) -> BaseConstructor:
        """
        Returns a PyYAML constructor.
        """
        return self._constructor

    @property
    def origin(self) -> Optional[str]:
        """
        Returns the origin path of the loading file if exists.
        """
        return self._origin


class Tag(ABC):
    @abstractmethod
    def construct(self, node: Node, context: TagContext) -> Any:
        """
        Construct the corresponding Python object to the given node.
        """


class Loader:
    def __init__(self):
        self._origin: Optional[str] = None

        class _Ctor(SafeConstructor):
            pass

        class _Loader(Reader, Scanner, Parser, Composer, _Ctor, Resolver):
            def __init__(self, stream):
                Reader.__init__(self, stream)
                Scanner.__init__(self)
                Parser.__init__(self)
                Composer.__init__(self)
                SafeConstructor.__init__(self)

        self._ctor = _Ctor
        self._Loader = _Loader

    def add_tag(self, name: str, tag: Tag) -> None:
        self._ctor.add_constructor(name, partial(self._apply_tag, tag))

    def load(self, stream: TextIO, origin: Optional[str] = None) -> object:
        """
        Parse the first YAML document in a stream and produce the corresponding Python object.
        """
        try:
            with self._on_origin(origin):
                return _load(stream, self._Loader)
        except MarkedYAMLError as error:
            raise YamlenError(cause=error)

    def load_from_path(self, path: str) -> object:
        """
        Parse the first YAML document in a file and produce the corresponding Python object.
        """
        origin = os.path.dirname(path)
        try:
            with open(path) as stream:
                return self.load(stream, origin)
        except FileNotFoundError as error:
            raise YamlenError(cause=error)

    def load_all(self, stream: TextIO, origin: Optional[str] = None) -> Iterator:
        """
        Parse all YAML documents in a stream and produce corresponding Python objects.
        """
        try:
            with self._on_origin(origin):
                yield from _load_all(stream, self._Loader)
        except MarkedYAMLError as error:
            raise YamlenError(cause=error)

    def load_all_from_path(self, path: str) -> Iterator:
        """
        Parse all YAML documents in a file and produce corresponding Python objects.
        """
        origin = os.path.dirname(path)
        try:
            with open(path) as stream:
                yield from self.load_all(stream, origin)
        except FileNotFoundError as error:
            raise YamlenError(cause=error)

    def _apply_tag(self, tag: Tag, ctor: BaseConstructor, node: Node) -> object:
        context = TagContext(loader=self, constructor=ctor, origin=self._origin)
        with on_node(node):
            return tag.construct(node, context)

    @contextmanager
    def _on_origin(self, origin: Optional[str]) -> Iterator:
        original = self._origin
        self._origin = origin
        try:
            yield
        finally:
            self._origin = original
